@@ -30,10 +30,11 @@ from nuitka import SourceCodeReferences
 from nuitka.__past__ import iter_modules
 from nuitka.containers.Namedtuples import makeNamedtupleClass
 from nuitka.containers.OrderedSets import OrderedSet
+from nuitka.Errors import NuitkaCodeDeficit
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonFlavors import isNuitkaPython
 from nuitka.PythonVersions import python_version
-from nuitka.Tracing import my_print, recursion_logger
+from nuitka.Tracing import recursion_logger
 from nuitka.tree.ReformulationMultidist import locateMultidistModule
 from nuitka.utils.AppDirs import getCacheDir
 from nuitka.utils.FileOperations import (
@@ -44,9 +45,9 @@ from nuitka.utils.FileOperations import (
 from nuitka.utils.Hashing import getFileContentsHash
 from nuitka.utils.Importing import (
     builtin_module_names,
+    getExtensionModuleSuffixes,
     getModuleFilenameSuffixes,
     getPackageDirFilename,
-    getSharedLibrarySuffixes,
     isBuiltinModuleName,
 )
 from nuitka.utils.ModuleNames import ModuleName
@@ -69,6 +70,9 @@ _debug_module_finding = None
 # Preference as expressed via --prefer-source-code
 _prefer_source_code_over_extension_modules = None
 
+# Do not add current directory to search path.
+_safe_path = None
+
 
 def setupImportingFromOptions():
     """Set up the importing layer from giving options."""
@@ -85,6 +89,9 @@ def setupImportingFromOptions():
         Options.shallPreferSourceCodeOverExtensionModules()
     )
 
+    global _safe_path
+    _safe_path = Options.hasPythonFlagNoCurrentDirectoryInPath()
+
     # Lets try and have this complete, please report failures.
     if Options.is_debug and not isNuitkaPython():
         _checkRaisingBuiltinComplete()
@@ -203,12 +210,46 @@ def isPackageDir(dirname):
         and os.path.isdir(dirname)
         and (
             python_version >= 0x300
-            or os.path.isfile(os.path.join(dirname, "__init__.py"))
             or isPreloadedPackagePath(dirname)
+            or getPackageDirFilename(dirname) is not None
         )
     )
 
 
+_package_module_name_cache = {}
+
+
+def isPackageModuleName(module_name):
+    """Decide if the give n module name is a package or not.
+
+    module, Cached as it involves disk access."""
+
+    if module_name not in _package_module_name_cache:
+        _module_name, module_filename, _module_kind, _finding = locateModule(
+            module_name=module_name,
+            parent_package=None,
+            level=0,
+        )
+
+        (
+            _main_added,
+            is_package,
+            _is_namespace,
+            _source_ref,
+            _source_filename,
+        ) = decideModuleSourceRef(
+            filename=module_filename,
+            module_name=module_name,
+            is_main=False,
+            is_fake=False,
+            logger=None,
+        )
+
+        _package_module_name_cache[module_name] = is_package
+
+    return _package_module_name_cache[module_name]
+
+
 def getModuleNameAndKindFromFilename(module_filename):
     """Given a filename, decide the module name and kind.
 
@@ -229,7 +270,10 @@ def getModuleNameAndKindFromFilename(module_filename):
     if module_filename.endswith(".pyc"):
         return ModuleName(os.path.basename(module_filename)[:-4]), "pyc"
 
-    for suffix in getSharedLibrarySuffixes():
+    # TODO: This sorting is necessary for at least Python2, but probably
+    # elsewhere too, need to be sure we follow the scan order of core Python
+    # here.
+    for suffix in sorted(getExtensionModuleSuffixes(), key=lambda suffix: -len(suffix)):
         if module_filename.endswith(suffix):
             return (
                 ModuleName(os.path.basename(module_filename)[: -len(suffix)]),
@@ -240,7 +284,7 @@ def getModuleNameAndKindFromFilename(module_filename):
         package_filename = getPackageDirFilename(module_filename)
 
         if package_filename is not None:
-            for suffix in getSharedLibrarySuffixes():
+            for suffix in getExtensionModuleSuffixes():
                 if package_filename.endswith(suffix):
                     return (
                         ModuleName(os.path.basename(module_filename)),
@@ -318,7 +362,7 @@ def normalizePackageName(module_name):
     return module_name
 
 
-def findModule(module_name, parent_package, level):
+def findModule(module_name, parent_package, level, logger):
     """Find a module with given package name as parent.
 
     The package name can be None of course. Level is the same
@@ -334,8 +378,8 @@ def findModule(module_name, parent_package, level):
 
     assert type(module_name) is ModuleName, module_name
 
-    if _debug_module_finding:
-        my_print(
+    if logger is not None:
+        logger.info(
             "findModule: Enter to search %r in package %r level %s."
             % (module_name, parent_package, level)
         )
@@ -382,13 +426,16 @@ def findModule(module_name, parent_package, level):
             return full_name.getPackageName(), module_filename, "py", "pth"
 
         try:
-            module_filename, module_kind = _findModule(module_name=full_name)
+            module_filename, module_kind = _findModule(
+                module_name=full_name,
+                logger=logger,
+            )
         except ImportError:
             # For relative import, that is OK, we will still try absolute.
             pass
         else:
-            if _debug_module_finding:
-                my_print(
+            if logger is not None:
+                logger.info(
                     "findModule: Relative imported module '%s' as '%s' in filename '%s':"
                     % (module_name, full_name, module_filename)
                 )
@@ -412,13 +459,16 @@ def findModule(module_name, parent_package, level):
             return package_name, module_filename, "py", "pth"
 
         try:
-            module_filename, module_kind = _findModule(module_name=module_name)
+            module_filename, module_kind = _findModule(
+                module_name=module_name,
+                logger=logger,
+            )
         except ImportError:
             # For relative import, that is OK, we will still try absolute.
             pass
         else:
-            if _debug_module_finding:
-                my_print(
+            if logger is not None:
+                logger.info(
                     "findModule: Found absolute imported module '%s' in filename '%s':"
                     % (module_name, module_filename)
                 )
@@ -439,7 +489,8 @@ ImportScanFinding = collections.namedtuple(
 # We put here things that are not worth it (Cython is not really used by
 # anything really, or where it's know to not have a big # impact, e.g. lxml.
 
-unworthy_namespaces = ("Cython", "lxml")
+# TODO: Do we really want this warning to persist?
+unworthy_namespaces = ("Cython", "lxml", "black", "tomli")
 
 
 def _reportCandidates(package_name, module_name, candidate, candidates):
@@ -488,7 +539,7 @@ def flushImportCache():
     module_search_cache.clear()
 
 
-def _findModuleInPath2(package_name, module_name, search_path):
+def _findModuleInPath2(package_name, module_name, search_path, logger):
     """This is out own module finding low level implementation.
 
     Just the full module name and search path are given. This is then
@@ -574,8 +625,8 @@ def _findModuleInPath2(package_name, module_name, search_path):
                 )
                 last_module_type = module_type
 
-    if _debug_module_finding:
-        my_print("Candidates:", candidates)
+    if logger is not None:
+        logger.info("Candidates: %r" % candidates)
 
     found_candidate = None
 
@@ -677,12 +728,12 @@ def getPackageSearchPath(package_name):
         return None
 
     if package_name is None:
-        result = (
-            [os.getcwd()]
-            + list(_main_paths)
-            + getPythonUnpackedSearchPath()
-            + list(_extra_paths)
-        )
+        result = []
+
+        if not _safe_path:
+            result.append(os.getcwd())
+
+        result += list(_main_paths) + getPythonUnpackedSearchPath() + list(_extra_paths)
     elif "." in package_name:
         parent_package_name, child_package_name = package_name.splitModuleBasename()
 
@@ -723,11 +774,13 @@ def getPackageSearchPath(package_name):
     )
 
 
-def _findModuleInPath(module_name):
+def _findModuleInPath(module_name, logger):
     package_name, module_name = module_name.splitModuleBasename()
 
-    if _debug_module_finding:
-        my_print("_findModuleInPath: Enter", module_name, "in", package_name)
+    if logger is not None:
+        logger.info(
+            "_findModuleInPath: Enter for %s in %s" % (module_name, package_name)
+        )
 
     # The "site" module must be located based on PYTHONPATH before it was
     # executed, while we normally search in PYTHONPATH after it was executed,
@@ -744,14 +797,18 @@ def _findModuleInPath(module_name):
 
     search_path = getPackageSearchPath(package_name)
 
-    if _debug_module_finding:
-        my_print(
-            "_findModuleInPath: Using search path", search_path, "for", package_name
+    if logger is not None:
+        logger.info(
+            "_findModuleInPath: Using search path %s for %s"
+            % (search_path, package_name)
         )
 
     try:
         module_filename, module_kind = _findModuleInPath2(
-            package_name=package_name, module_name=module_name, search_path=search_path
+            package_name=package_name,
+            module_name=module_name,
+            search_path=search_path,
+            logger=logger,
         )
     except SyntaxError:
         # Warn user, as this is kind of unusual.
@@ -762,8 +819,11 @@ def _findModuleInPath(module_name):
 
         return None, None
 
-    if _debug_module_finding:
-        my_print("_findModuleInPath: _findModuleInPath2 gave", module_filename)
+    if logger is not None:
+        logger.info(
+            "_findModuleInPath: _findModuleInPath2 gave %s %s"
+            % (module_filename, module_kind)
+        )
 
     return module_filename, module_kind
 
@@ -771,12 +831,12 @@ def _findModuleInPath(module_name):
 module_search_cache = {}
 
 
-def _findModule(module_name):
+def _findModule(module_name, logger):
     # Not a good module name. TODO: Push this to ModuleName() creation maybe.
     assert module_name != ""
 
-    if _debug_module_finding:
-        my_print("_findModule: Enter to search '%s'." % (module_name,))
+    if logger is not None:
+        logger.info("_findModule: Enter to search '%s'." % module_name)
 
     assert module_name.getBasename(), module_name
 
@@ -785,8 +845,8 @@ def _findModule(module_name):
     if key in module_search_cache:
         result = module_search_cache[key]
 
-        if _debug_module_finding:
-            my_print("_findModule: Cached result (see previous call).")
+        if logger is not None:
+            logger.info("_findModule: Cached result (see previous call).")
 
         if result is ImportError:
             raise ImportError
@@ -794,7 +854,10 @@ def _findModule(module_name):
         return result
 
     try:
-        module_search_cache[key] = _findModuleInPath(module_name)
+        module_search_cache[key] = _findModuleInPath(
+            module_name=module_name,
+            logger=logger,
+        )
     except ImportError:
         module_search_cache[key] = ImportError
         raise
@@ -804,7 +867,7 @@ def _findModule(module_name):
     return module_search_cache[key]
 
 
-def locateModule(module_name, parent_package, level):
+def locateModule(module_name, parent_package, level, logger=None):
     """Locate a module with given package name as parent.
 
     The package name can be None of course. Level is the same
@@ -817,13 +880,22 @@ def locateModule(module_name, parent_package, level):
         kind.
     """
 
+    if not _main_paths:
+        raise NuitkaCodeDeficit(
+            "Error, cannot locate modules before import mechanism is setup."
+        )
+
     if module_name.isMultidistModuleName():
         return locateMultidistModule(module_name)
 
+    if _debug_module_finding and logger is None:
+        logger = recursion_logger
+
     module_package, module_filename, module_kind, finding = findModule(
         module_name=module_name,
         parent_package=parent_package,
         level=level,
+        logger=logger,
     )
 
     # Allowing ourselves to be lazy.
@@ -936,45 +1008,64 @@ def decideModuleSourceRef(filename, module_name, is_main, is_fake, logger):
     )
 
 
-# spell-checker: ignore _posixsubprocess,pyexpat,xxsubtype
-
+# spell-checker: ignore _posixsubprocess,pyexpat,xxsubtype,_bytesio,_interpchannels
+# spell-checker: ignore _interpqueues,_lsprof,_multibytecodec,_posixshmem _winapi
+# spell-checker: ignore  _testbuffer _testexternalinspection _testimportmultiple
+# spell-checker: ignore _testinternalcapi _testmultiphase _testsinglephase
+# spell-checker: ignore _xxtestfuzz _xxsubinterpreters imageop _xxinterpchannels
 _stdlib_module_raises = {
-    "__builtin__": False,
     "_abc": False,
+    "__builtin__": False,
     "_ast": False,
+    "_asyncio": False,
     "_bisect": False,
     "_blake2": False,
     "_bytesio": False,
-    "_codecs": False,
+    "_bz2": False,
     "_codecs_cn": False,
     "_codecs_hk": False,
     "_codecs_iso2022": False,
     "_codecs_jp": False,
     "_codecs_kr": False,
     "_codecs_tw": False,
+    "_codecs": False,
     "_collections": False,
     "_contextvars": False,
+    "_crypt": False,
     "_csv": False,
+    "_ctypes": False,
+    "_ctypes_test": False,
+    "_curses": False,
+    "_curses_panel": False,
     "_datetime": False,
+    "_dbm": False,
+    "_decimal": False,
     "_elementtree": False,
     "_fileio": False,
     "_functools": False,
+    "_hashlib": False,
     "_heapq": False,
     "_hotshot": False,
     "_imp": False,
+    "_interpchannels": False,
+    "_interpqueues": False,
+    "_interpreters": False,
     "_io": False,
     "_json": False,
     "_locale": False,
     "_lsprof": False,
+    "_lzma": False,
     "_md5": False,
+    "_multiprocessing": False,
     "_multibytecodec": False,
     "_opcode": False,
     "_operator": False,
     "_peg_parser": False,
     "_pickle": False,
+    "_posixshmem": False,
     "_posixsubprocess": False,
+    "_queue": False,
     "_random": False,
-    "_subprocess": False,
     "_sha": False,  # TODO: Not entirely clear if that's true
     "_sha1": False,
     "_sha2": False,
@@ -983,29 +1074,45 @@ _stdlib_module_raises = {
     "_sha512": False,
     "_signal": False,
     "_socket": False,
+    "_sqlite3": False,
     "_sre": False,
     "_stat": False,
     "_statistics": False,
+    "_ssl": True,
     "_string": False,
     "_struct": False,
+    "_subprocess": False,
+    "_suggestions": False,
     "_symtable": False,
+    "_sysconfig": False,
+    "_testbuffer": False,
+    "_testexternalinspection": False,
+    "_testimportmultiple": False,
+    "_testinternalcapi": False,
+    "_testmultiphase": False,
+    "_testsinglephase": False,
     "_thread": False,
-    "_tracemalloc": False,
+    "_tkinter": True,
     "_tokenize": False,
+    "_tracemalloc": False,
     "_typing": False,
+    "_uuid": False,
     "_warnings": False,
     "_weakref": False,
     "_winapi": False,
     "_winreg": False,
+    "_xxtestfuzz": False,
     "_xxsubinterpreters": False,
+    "_xxinterpchannels": False,
+    "_zoneinfo": False,
     "array": False,
     "atexit": False,
     "audioop": False,
     "binascii": False,
     "builtins": False,
-    "cmath": False,
-    "cStringIO": False,
     "cPickle": False,
+    "cStringIO": False,
+    "cmath": False,
     "datetime": False,
     "errno": False,
     "exceptions": False,
@@ -1014,40 +1121,37 @@ _stdlib_module_raises = {
     "future_builtins": False,
     "gc": False,
     "grp": False,
-    "itertools": False,
     "imageop": False,
     "imp": False,
-    "operator": False,
+    "itertools": False,
     "marshal": False,
     "math": False,
     "mmap": False,
     "msvcrt": False,
+    "nis": False,
     "nt": False,
+    "operator": False,
+    "ossaudiodev": False,
     "parser": False,
     "posix": False,
     "pwd": False,
     "pyexpat": False,
+    "readline": True,
+    "resource": False,
     "select": False,
     "signal": False,
-    "strop": False,
     "spwd": False,
+    "strop": False,
     "sys": False,
     "syslog": False,
-    "time": False,
+    "termios": False,
     "thread": False,
+    "time": False,
     "unicodedata": False,
     "winreg": False,
     "xxsubtype": False,
     "zipimport": False,
     "zlib": False,
-    "_ssl": True,
-    "_xxinterpchannels": False,
-    # TODO: 3.13 clarify if this is actually true
-    "_interpchannels": False,
-    "_interpreters": False,
-    "_interpqueues": False,
-    "_sysconfig": False,
-    "_suggestions": False,
 }
 
 
@@ -1071,7 +1175,7 @@ def _getChildPackageNames(module_name):
         child_name = module_name.getChildNamed(module_info.name)
         yield child_name
 
-        if module_info.ispkg:
+        if module_info.ispkg:  # spell-checker: ignore ispkg
             for sub_module_name in getChildPackageNames(child_name):
                 yield sub_module_name
 
