@@ -34,7 +34,9 @@ from nuitka.Options import (
     getCompilationReportFilename,
     getCompilationReportTemplates,
     getCompilationReportUserData,
+    getOnefileTempDirSpec,
     isOnefileMode,
+    isOnefileTempDirMode,
     shallCreateDiffableCompilationReport,
 )
 from nuitka.OutputDirectories import (
@@ -56,6 +58,7 @@ from nuitka.utils.Distributions import (
     getDistributionName,
     getDistributionsFromModuleName,
     getDistributionVersion,
+    isDistributionVendored,
 )
 from nuitka.utils.FileOperations import (
     getReportPath,
@@ -161,6 +164,14 @@ def _getReportInputData(aborted):
         for dist in all_distributions
     )
 
+    module_distribution_vendored = dict(
+        (
+            getDistributionName(dist),
+            isDistributionVendored(getDistributionName(dist)),
+        )
+        for dist in all_distributions
+    )
+
     module_exclusions = dict((module.getFullName(), {}) for module in getDoneModules())
 
     # TODO: The module filename, and other things can be None. Once we change to
@@ -714,13 +725,21 @@ def writeCompilationReport(report_filename, report_input_data, diffable):
             user_enabled="no" if plugin.isAlwaysEnabled() else "yes",
         )
 
+    if isOnefileMode():
+        _onefile_xml_node = TreeXML.appendTreeElement(
+            root,
+            "onefile",
+            cache_mode="temporary" if isOnefileTempDirMode() else "cached",
+            unpack_dir=getOnefileTempDirSpec(),
+        )
+
     distributions_xml_node = TreeXML.appendTreeElement(
         root,
         "distributions",
     )
 
     for distribution in report_input_data["all_distributions"]:
-        TreeXML.appendTreeElement(
+        distribution_node = TreeXML.appendTreeElement(
             distributions_xml_node,
             "distribution",
             name=getDistributionName(distribution),
@@ -730,6 +749,11 @@ def writeCompilationReport(report_filename, report_input_data, diffable):
             ],
         )
 
+        if report_input_data["module_distribution_vendored"][
+            getDistributionName(distribution)
+        ]:
+            distribution_node.attrib["vendored"] = "yes"
+
     python_xml_node = TreeXML.appendTreeElement(
         root,
         "python",
